# ifndef UTILS_H
# define UTILS_H

# include <stdexcept>
# include <iostream>
# include <string>
# include <sstream>

struct error : std::runtime_error
{
  error(const std::string &desc) : std::runtime_error(desc.c_str()) {}
};

inline void check(const char *msg, bool cond)
{
  if(!cond) throw error(msg);
}

struct counted_ptr_man
{
  template <typename T>
  counted_ptr_man(T *p) : p((void *)p), deleter(delete_function<T>), count(1) {}
  void *const p;
  void (*const deleter)(void *);
  int count;
private:
  template <typename T>
  static void delete_function(void *p) { delete (T *)p; }
};

template <typename Ty>
struct counted_ptr
{
  counted_ptr(Ty *p)
  {
    try {
      pman = new counted_ptr_man(p);
      //std::cout << "new counted_ptr this=" << this << ": pman=" <<  pman << "\n";
    } catch(...) {
      delete p;
      throw;
    }
  }
  template <typename Other>
  counted_ptr(const counted_ptr<Other> &other)
    :pman(other.pman)
  {
    //std::cout << "share counted_ptr this=" << this << ": pman=" <<  pman << "; count=" << pman->count << "\n";
    sizeof(*(Ty **)0 = (Other *)0);
    if(pman)
      pman->count++;
  }
  counted_ptr(const counted_ptr &other)
    :pman(other.pman)
  {
    //std::cout << "copy counted_ptr this=" << this << ": other=" << &other << "; pman=" <<  pman << std::flush;
    if(pman)
      pman->count++;
    //std::cout << "; count=" << pman->count << "\n";
  }
  counted_ptr() : pman(0) {}
  Ty &operator*() const { return *operator->(); }
  Ty *operator->() const { check("counted_ptr: null reference", pman); return (Ty *)pman->p; }
  Ty *get() const { return pman ? (Ty *)pman->p : 0; }
  bool operator!() const { return !pman; }
  template <typename Other>
  counted_ptr &operator=(const counted_ptr<Other> &other)
  {
    sizeof(*(Ty **)0 = (Other *)0);
    copy_from(other);
    return *this;
  }
  counted_ptr &operator=(const counted_ptr &other)
  {
    copy_from(other);
    return *this;
  }
  void release()
  {
    //std::cout << "release counted_ptr this=" << this << ": pman=" <<  pman;
    //if(pman) std::cout << "; count=" << pman->count;
    //std::cout << "\n";
    if(pman && !--pman->count)
    {
      pman->deleter(pman->p);
      delete pman;
    }
    pman = 0;
  }
  ~counted_ptr() { release(); }
private:
  template <typename Other>
  void copy_from(const counted_ptr<Other> &other)
  {
    if(&other != this)
    {
      release();
      pman = other.pman;
      if(pman)
        pman->count++;
    }
  }
  counted_ptr_man *pman;
};

# define DEF_RUNTIME_ERROR(name)                                    \
  struct name : ::std::runtime_error                                \
  {                                                                 \
    name() : ::std::runtime_error(#name) {}                         \
  };                                                                \
  /**/

struct streamer
{
  streamer() {}
  std::string str() const { return ost.str(); }
  template <typename T>
  const streamer &operator<<(const T &x) const
  {
    ost << x;
    return *this;
  }
private:
  mutable std::ostringstream ost;
  streamer(const streamer &);
};

# define STR(chain) (streamer() << chain).str()

# define ARRAY_SIZE(arr) ((int)(sizeof (arr) / sizeof *(arr)))

template <typename Bi>
Bi prior(Bi it)
{
  return --it;
}

inline void attention() {}

# endif
